#include "CIS.h"

/**
 * 	Calculate Clusters in the graph using the Connected Iterative Scan algorithm.
 */
template <typename R>
void IterativeDensity<R>::calculate()
{ 
    //In the case a seed gets disconnected, take the best connected component and continue
    future_actions* act = new take_best(0.000001);

    unsigned int uid;
    bidirected_t::map::iterator mip;
    std::set<seed::vertex_set> allClust;
    
    //MIP iterates over all vertices in the graph one at a time, using them for seeds.
    int seedCount = 0;
    int tenth = map.size() / 10;
    int epoch = 0;
    for (mip = map.begin(); mip!=map.end(); mip++)
    {
      if (tenth != 0 && seedCount % tenth == 0){
	cerr << epoch * 10 << " Percent Done." << endl;
	epoch++;
      }
      
      /*Have to retrieve the correct representation of the vertex*/
	uid = mip->first;
        bidirected_t::vertex v = map.find(uid)->second;
	
        seed input(this->dens, graph, v); 	//Create Seed
	
        std::vector<seed>* clusters = cluster(input, act, &graph);

	/*After clustering is done, there should be a single, connected component.*/
        if (clusters->size() != 1)
        {
            std::cerr << "Broken cluster run\n";
            return;
        }

        seed output = (*clusters)[0];	
        delete clusters;

	/*Check for duplicates before printing and saving*/
	bool checker = (output.vertices()).size() > 2;
	if (allClust.find(output.vertices()) == allClust.end() && checker)
	{
	  allClust.insert(output.vertices());
	  /*Auto print - no save*/
	  if (this->translate){
	    print_set_trans<bidirected_t>(graph, output.vertices(), *(this->fileout), output.dens(), this->idmap, delimiters[0]);
	  } else {
	    print_set<bidirected_t>(graph, output.vertices(), *(this->fileout), output.dens());
	  }
	}
	
	++seedCount;
    }
    
    allClust.clear();
}

template void IterativeDensity<double>::calculate();
template void IterativeDensity<int>::calculate();
