#ifndef H_MYSET
#define H_MYSET

#include <iostream>
#include <set>

using namespace std;

/**
 *@fn template <typename T> T Intersection(T C1, T C2)
 *
 * Get the intersection of two containers
 *
 * @param C1 Container 1
 * @param C2 Container 2
 */
template < typename T >
T Intersection(T C1, T C2){
  
  typename T::iterator it_set;
  T ret;
  
  for (it_set = C1.begin(); it_set != C1.end(); it_set++){
    if (find(C2.begin(), C2.end(), *it_set) != C2.end())
      ret.insert(ret.begin(), *it_set);
  }
  
  return ret;
};

/**
 * @fn template < typename T > T Difference (T C1, T C2)
 *
 * Get the difference of two containers (C1 - C2)
 *
 * @param C1 Container 1
 * @param C2 Container 2
 */
template < typename T >
T Difference(T C1, T C2){
  
  typename T::iterator it_set;
  T ret;
  
  for (it_set = C1.begin(); it_set != C1.end(); it_set++){
    if (find(C2.begin(), C2.end(), *it_set) == C2.end())
      ret.insert(ret.begin(), *it_set);
  }
  
  return ret;
};

/**
 * @fn template <typename T> T Union (T C1, T C2)
 * Get the union of two containers. The order of returned elements is in reverse first seen order, considering C1 first, then C2.
 *
 * @param C1 Container 1
 * @param C2 Container 2
 */
template <typename T>
T Union(T C1, T C2){
  typename T::iterator it_set;
  T ret;
  
  for (it_set = C1.begin(); it_set != C1.end(); it_set++){
    if (find(ret.begin(), ret.end(), *it_set) == ret.end())
      ret.insert(ret.begin(), *it_set);
  }
  
  for (it_set = C2.begin(); it_set != C2.end(); it_set++){
    if (find(ret.begin(), ret.end(), *it_set) == ret.end())
      ret.insert(ret.begin(), *it_set);
  }
  
  return ret;
}
#endif
