#ifndef _Hidden_Groups_CPP_
#define _Hidden_Groups_CPP_

#include <vector>
#include <map>
#include "Basic_structures.cpp"

using namespace std;

///////////////Matching Algorithms////////////////////////////

//times_list contains lists of all communications between consecutive people in a chain
//ex: for chain A -> B -> C the first list in times_list contains the times for all communications from A to B
//Algorithm_Chain finds the maximum matching between times in time_list such that the minimum and maximum constraints are satisfied.
vector<pair<double, double> >* Algorithm_Chain(pair<vector<double>*, vector<double>* >& times_list, double minimum, double maximum){
    vector<pair<double, double> >* matches = new  vector<pair<double, double> >;
    if (times_list.first -> empty() || times_list.second -> empty()){return matches;}
    pair<int, int> current_match; current_match.first = 0; current_match.second = 0;
    //bool end = false;
    while(true){
        double diff = (*times_list.second)[current_match.second] - (*times_list.first)[current_match.first];
        if(diff < minimum){
            current_match.second++;
            if(current_match.second >= times_list.second -> size()){break;}
        }
        else if (diff <= maximum && diff >= minimum){
            pair<double, double> p;
            p.first = (*times_list.first)[current_match.first];
            p.second = (*times_list.second)[current_match.second];
            matches -> push_back(p);
            current_match.first++;
            current_match.second++;
            if(current_match.first >= times_list.first -> size() ||
               current_match.second >= times_list.second -> size()){break;}
        }
        else{
            current_match.first++;
            if(current_match.first >= times_list.first -> size()){break;}
        }
    }
    return matches;
}

//Finds a maximum matching for times_list that satisfies the constraint defined by delta
vector<pair<double, double> >* Algorithm_Sibling(pair<vector<double>*, vector<double>* >& times_list, double delta){
    vector<pair<double, double> >* matches = new  vector<pair<double, double> >;
    if (times_list.first -> empty() || times_list.second -> empty()){return matches;}
    pair<int, int> current_match; current_match.first = 0; current_match.second = 0;
    while(true){
        double diff = (*times_list.second)[current_match.second] - (*times_list.first)[current_match.first];
        if(diff < -1*delta){
            current_match.second++;
            if(current_match.second >= times_list.second -> size()){break;}
        }
        else if (diff <= delta && diff >= -1*delta){
            pair<double, double> p;
            p.first = (*times_list.first)[current_match.first];
            p.second = (*times_list.second)[current_match.second];
            matches -> push_back(p);
            current_match.first++;
            current_match.second++;
            if(current_match.first >= times_list.first -> size() ||
               current_match.second >= times_list.second -> size()){break;}
        }
        else{
            current_match.first++;
            if(current_match.first >= times_list.first -> size()){break;}
        }
    }
    return matches;
}

vector<pair<double, double> >* Ordered_Algorithm_Sibling(pair< vector<double>*, vector<double>* >& times_list, double delta){
        return Algorithm_Chain(times_list, 0, 2*delta);
}


//////////////////////////Finding Triples/////////////////////////////////

//Assembles a 'speaker_list' such that at each index, i, there is a map so that
//given the key, j, the map yields a vector of all times i communicated with j
typedef vector<map<int, vector<double> > > speaker_list;
speaker_list Get_speaker_list(vector<communication>& comms){
    speaker_list alist;
    map<int, vector<double> > m;
    for(int i=0; i<comms.size(); i++){
        while(comms[i].speaker >= alist.size() || comms[i].listener >= alist.size()){alist.push_back(m);}
        (alist[comms[i].speaker][comms[i].listener]).push_back(comms[i].time);
    }
    return alist;
}

//Find_Significan_Triples is a helper function called by Significant_Frequencies and Significant_Matches
//It Finds every valid triple in comms that occurs at least 'frequency' times, based on the given minimum, maximum, and delta.
typedef map<triple, vector<pair<double, double> >* > triple_match_map;
void Find_Significant_Triples(speaker_list& slist, double minimum, double maximum, double delta, int frequency, triple_match_map*& final_matches, map<triple, int>*& frequency_map,  bool store_matches = true, string triple_type = "both"){
   // speaker_list slist = Get_speaker_list(comms);
    if(store_matches){final_matches = new triple_match_map;}
    else{frequency_map = new map<triple, int>;}
    for(int i=0; i<slist.size(); i++){
        map<int,vector<double> >::iterator itr = slist[i].begin();
            while(itr != slist[i].end()){
                map<int,vector<double> >::iterator itr2;
                if(triple_type != "sibling"){ //ie triple_type equals "both" or "chain"
                    //finds all significant chain triples from person-i
                    itr2 = (slist[(itr -> first)]).begin(); //itr2 represents all people itr has communicated with
                     while(itr2 != (slist[(itr -> first)]).end()){
                        if(itr2 -> first == i){itr2++; continue;}
                        pair<vector<double>*, vector<double>* > times;
                        times.first = &(itr -> second);
                        times.second = &(itr2 -> second);
                        vector<pair<double, double> >* matches = Algorithm_Chain(times, minimum, maximum);
                        if((*matches).size() >= frequency){
                            triple t;
                            t.a = i;
                            t.b = itr -> first;
                            t.c = itr2 -> first;
                            t.chain = true;
                            if(store_matches){(*final_matches)[t] = matches;}
                            else {(*frequency_map)[t] = matches -> size(); delete matches;}
                        }
                        else {delete matches;}
                        itr2++;
                     }
                }
                if(triple_type != "chain"){ // triple_type equals "both" or "sibling"
                     //finds all significant sibling triples from person-i
                     itr2=slist[i].begin(); //itr2 is used to consider every possible pair of two people person i has spoken to.
                     while(itr2 != itr){
                        if(itr2 -> first == i){itr2++; continue;}
                        pair<vector<double>*, vector<double>* > times;
                        times.first = &(itr -> second);
                        times.second = &(itr2 -> second);
                        vector<pair<double, double> >* matches = Algorithm_Sibling(times, delta);
                        if((*matches).size() >= frequency){
                            triple t;
                            t.a = i;
                            t.b = itr -> first;
                            t.c = itr2 -> first;
                            t.chain = false;
                            if(store_matches){(*final_matches)[t] = matches;}
                            else{(*frequency_map)[t] = matches -> size(); delete matches;}
                        }
                        else{delete matches;}
                        itr2++;
                     }
                }
            itr++;
        }
    }
}

//This operator is used by the triple_match_map returned by Significant_Matches
bool operator< (const triple& t1, const triple& t2){
    if(t1.a < t2.a){return true;}
    else if(t1.a == t2.a){
        if(t1.b < t2.b){return true;}
        else if(t1.b == t2.b){
            if(t1.c < t2.c){return true;}
            else if(t1.c == t2.c  && t1.chain > t2.chain){return true;}
        }
    }
    return false;
}

//Significant_Matches returns a map from triples onto a vector<pair<double, double>>
//This vector stores every pair of times the given triple occured.
//Note that this map uses significantly more memory than the map returned by Significant_Frequencies.
triple_match_map* Significant_Matches(vector<communication>& comms, double minimum, double maximum, double delta, int frequency, string triple_type = "both"){
    triple_match_map* return_pointer;
    map<triple, int>* temp_pointer;
    speaker_list slist = Get_speaker_list(comms);
    Find_Significant_Triples(slist, minimum, maximum, delta, frequency, return_pointer, temp_pointer, true, triple_type);
    return return_pointer;
}

//Performs same function as Significant_Frequencies (below), but saves time if slist was already calculated
map<triple, int>* Significant_Frequencies_with_Speakers(speaker_list& slist, double minimum, double maximum, double delta, int frequency, string triple_type = "both"){
    map<triple, int>* return_pointer;
    triple_match_map* temp_pointer;
    Find_Significant_Triples(slist, minimum, maximum, delta, frequency, temp_pointer, return_pointer, false, triple_type);
    return return_pointer;
}

//Significant_Frequencies returns a map from triples onto an integer
//The integer represents the number of times that triple occured, without storing the actual matches.
map<triple, int>* Significant_Frequencies(vector<communication>& comms, double minimum, double maximum, double delta, int frequency, string triple_type = "both"){
    speaker_list slist = Get_speaker_list(comms);
    return Significant_Frequencies_with_Speakers(slist, minimum, maximum, delta, frequency, triple_type);
}

#endif
