#include <fstream>
#include <math.h>

#include "Hidden_Groups.cpp"
#include "constrained_random_data.cpp"

using namespace std;

double Find_Standard_Dev(vector<int>& values, int sum){
    double average = (double)sum/(double)(values.size());
    double squares = 0;
    for(int i=0; i<values.size(); i++){
        squares += pow((values[i] - average),2);
    }
    return pow(squares / values.size(), 0.5);
}

//takes in the sythetic and actual data and the std-devs. Calculates a good significant frequency to be used on the actual data.
int get_Frequency(vector<int> actual_data, vector<double> synthetic_data, vector<int> standard_devs, int num_devs){
       //increases synthetic_data by num_devs standard deviations.
        for(int i=1; i<synthetic_data.size(); i++){
            synthetic_data[i] += standard_devs[i] * num_devs;
        }
       //Steps backwards from end of list
       //finds first point where the actually frequency is less than the synthetic frequency
        int index = min(synthetic_data.size(), actual_data.size()) - 1;
        while(index > 0){
            if(actual_data[index] < synthetic_data[index]){break;}
            index--;
        }
        return index + 1;
}

void Get_synthetic_data(vector<communication> comms, vector<double>& synthetic_data, vector<int>& std_dev_list, double minimum, double maximum, double delta, int total_repeats = 1, string triple_type = "both"){
   //finds frequencies of all the triples
    vector<vector<int> > frequency_list;
    for(int i=0; i<total_repeats; i++){

        //gets significant triples from random communications
        speaker_list slist;
        Constrained_Random_Data(comms, slist);
        map<triple, int>* t_map = Significant_Frequencies_with_Speakers(slist, minimum, maximum, delta, 1);

        //Adds new significant triples to frequency_list
        map<triple, int>::iterator itr = t_map -> begin();
        vector<int> temp(total_repeats, 0);
        vector<int> sums;
        while(itr != t_map -> end()){
            int current_frequency = itr -> second;
            while (frequency_list.size() <= current_frequency){
                frequency_list.push_back(temp);
                synthetic_data.push_back(0);
            }
            frequency_list[current_frequency][i]++;
            itr++;
        }

        //Reorganizes frequency_list so that the value at frequency_list[j][i]
        // is the number of triples of *at least* j (possibly greater)
        int remaining_total = t_map -> size();
        for(int j=1; j<frequency_list.size(); j++){
            int t = frequency_list[j][i];
            frequency_list[j][i] = remaining_total;
            synthetic_data[j] += remaining_total;
            remaining_total -= t;
        }
        delete t_map;

    }

    //calclulates standard deviation for each triple-frequency
    std_dev_list.resize(frequency_list.size());
    for(int i=1; i<frequency_list.size(); i++){
        std_dev_list[i] = Find_Standard_Dev(frequency_list[i], synthetic_data[i]);
        synthetic_data[i] = (double)((double)synthetic_data[i] / (double)(total_repeats));
    }
}
